{**********************************************}
{                                              }
{     FastReport 2.5 - SQLDirect components    }
{             MacroQuery component             }
{                                              }
{   Copyright (c) 1997-2007 by Yuri Sheino.    }
{   Copyright (c) 1998-2007 by Tzyganenko A.   }
{ Copyright (c) 2001-2007 by Stalker SoftWare  }
{                                              }
{**********************************************}

unit FR_SDMacroQuery;

interface

{$I FR.inc}
{$I FR_SD.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, StdCtrls, Controls, Forms,
  Menus, Dialogs, FR_Class, FR_Pars, DB, FR_SDCommon, FR_DBUtils, SDEngine,
  TypInfo;

type
  TfrSDMacroQuery = class(TfrSDDataSet)
  private
    FSDMacroQuery: TSDMacroQuery;
    FParams      :TfrVariables;
    FMacroParams :TfrVariables;
    procedure SQLEditor(Sender: TObject);
    procedure ParamsEditor(Sender: TObject);
    procedure MacroParamsEditor(Sender: TObject);
    procedure ReadParams(Stream: TStream);
    procedure WriteParams(Stream: TStream);
    procedure ReadMacroParams(Stream: TStream);
    procedure WriteMacroParams(Stream: TStream);
    function GetParamKind(Index: Integer): TfrParamKind;
    procedure SetParamKind(Index: Integer; Value: TfrParamKind);
    function GetParamText(Index: Integer): String;
    procedure SetParamText(Index: Integer; Value: String);
    procedure BeforeOpenQuery(DataSet: TDataSet);
    function GetMacroParamText(Index: Integer): String;
    procedure SetMacroParamText(Index: Integer; const Value: String);
  protected
    procedure SetPropValue(Index: String; Value: Variant); override;
    function GetPropValue(Index: String): Variant; override;
    function DoMethod(MethodName: String; Par1, Par2, Par3: Variant): Variant; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure LoadFromStream(Stream: TStream); override;
    procedure SaveToStream(Stream: TStream); override;
    procedure DefineProperties; override;
    procedure Loaded; override;
    procedure SaveToFR3Stream(Stream: TStream); override;
    property Query: TSDMacroQuery read FSDMacroQuery;
    property ParamKind[Index: Integer]: TfrParamKind read GetParamKind write SetParamKind;
    property ParamText[Index: Integer]: String read GetParamText write SetParamText;
    property MacroParamText[Index: Integer]: String read GetMacroParamText write SetMacroParamText;
  end;


implementation

uses
  FR_Utils, FR_Const, FR_DBSQLEdit, FR_SDParam,
{$IFDEF SD3X}
  FR_SDMacroParam,
{$ENDIF}
  FR_SDQuery
{$IFDEF QBUILDER}
, FR_SDQB, QBuilder
{$ENDIF}
{$IFDEF Delphi6}
, Variants
{$ENDIF};

{ TfrSDMacroQuery }

constructor TfrSDMacroQuery.Create;
begin

 inherited Create;

 FSDMacroQuery := TSDMacroQuery.Create(frDialogForm);
 FSDMacroQuery.BeforeOpen := BeforeOpenQuery;
 FDataSet := FSDMacroQuery;
 FDataSource.DataSet := FDataSet;

 FParams := TfrVariables.Create;
 FMacroParams := TfrVariables.Create;

 Component := FSDMacroQuery;
 BaseName := 'SDMacroQuery';
 Bmp.LoadFromResourceName(hInstance, 'FR_SDMACROQUERY');

end; { Create }

destructor TfrSDMacroQuery.Destroy;
begin
 FParams.Free;
 FMacroParams.Free;
 inherited Destroy;
end; { Destroy }

procedure TfrSDMacroQuery.DefineProperties;

 function GetMasterSource: String;
 var
   i: Integer;
   sl: TStringList;

 begin

   Result := '';
   sl := TStringList.Create;
   frGetComponents(FSDMacroQuery.Owner, TDataSet, sl, FSDMacroQuery);
   sl.Sort;
   for i := 0 to sl.Count - 1 do
     Result := Result + sl[i] + ';';
   sl.Free;

 end; { GetMasterSource }

begin

 inherited DefineProperties;

 AddEnumProperty('DataSource', GetMasterSource, [Null]);
 AddProperty('ParamCheck', [frdtBoolean], nil);
 AddProperty('Params', [frdtHasEditor], ParamsEditor);
 AddProperty('SQL', [frdtHasEditor], SQLEditor);
 AddProperty('SQL.Count', [], nil);
 AddProperty('RequestLive', [frdtBoolean], nil);
 AddProperty('MacroChar', [frdtString], nil);
 AddProperty('Macros', [frdtHasEditor], MacroParamsEditor);
 AddProperty('GetMacro',[],nil);
 AddProperty('SetMacro',[],nil);


end; { DefineProperties }

procedure TfrSDMacroQuery.SetPropValue(Index: String; Value: Variant);
var
  d: TDataSet;

begin

 inherited SetPropValue(Index, Value);
 Index := AnsiUpperCase(Index);

 if Index = 'DATASOURCE' then begin
   d := frFindComponent(FSDMacroQuery.Owner, Value) as TDataSet;
   FSDMacroQuery.DataSource := frGetDataSource(FSDMacroQuery.Owner, d);
 end
 else
 if Index = 'PARAMCHECK' then
   FSDMacroQuery.ParamCheck := Value
 else
 if Index = 'SQL' then
   FSDMacroQuery.SQL.Text := Value
 else
 if Index = 'REQUESTLIVE' then
   FSDMacroQuery.RequestLive := Value
 else
 if Index = 'MACROCHAR' then begin
   if Value <> '' then
     FSDMacroQuery.MacroChar := VarToStr(Value)[1]
 end;

end; { SetPropValue }

function TfrSDMacroQuery.GetPropValue(Index: String): Variant;
begin

 Index := AnsiUpperCase(Index);
 Result := inherited GetPropValue(Index);
 if Result <> Null then Exit;

 if Index = 'DATASOURCE' then
   Result := frGetDataSetName(FSDMacroQuery.Owner, FSDMacroQuery.DataSource)
 else
 if Index = 'PARAMCHECK' then
   Result := FSDMacroQuery.ParamCheck
 else
 if Index = 'SQL' then
   Result := FSDMacroQuery.SQL.Text
 else
 if Index = 'SQL.COUNT' then
   Result := FSDMacroQuery.SQL.Count
 else
 if Index = 'REQUESTLIVE' then
   Result := FSDMacroQuery.RequestLive
 else
 if Index = 'MACROCHAR' then
   Result := FSDMacroQuery.MacroChar

end; { GetPropValue }

function TfrSDMacroQuery.DoMethod(MethodName: String; Par1, Par2, Par3: Variant): Variant;
begin

 Result := inherited DoMethod(MethodName, Par1, Par2, Par3);

 if Result = Null then
   Result := LinesMethod(FSDMacroQuery.SQL, MethodName, 'SQL', Par1, Par2, Par3);

 if MethodName = 'EXECSQL' then begin
   BeforeOpenQuery(FSDMacroQuery);
   FSDMacroQuery.ExecSQL
 end else
 if MethodName = 'SETMACRO' then begin
   if Assigned(FSDMacroQuery.Macros.FindParam(frParser.Calc(Par1))) then
     FMacroParams[frParser.Calc(Par1)] := Par2
 end else
 if MethodName = 'GETMACRO' then begin
   if Assigned(FSDMacroQuery.Macros.FindParam(frParser.Calc(Par1))) then
     Result := FMacroParams[frParser.Calc(Par1)]
   else
     Result := ''
 end

end; { DoMethod }

procedure TfrSDMacroQuery.LoadFromStream(Stream: TStream);
var
  s: String;

begin
 
 FFixupList.Clear;
 inherited LoadFromStream(Stream);

 FSDMacroQuery.DatabaseName := frReadString(Stream);
 FSDMacroQuery.SessionName := frReadString(Stream);
 FSDMacroQuery.AutoCalcFields := frReadBoolean(Stream);

 s := frReadString(Stream);
 FFixupList['DataSource'] := s;
 Prop['DataSource'] := FFixupList['DataSource'];

 if frReadBoolean(Stream) then
   FSDMacroQuery.Options := FSDMacroQuery.Options+[doDetachOnFetchAll]
 else
   FSDMacroQuery.Options := FSDMacroQuery.Options-[doDetachOnFetchAll];

 FSDMacroQuery.ParamCheck := frReadBoolean(Stream);
 frReadMemo(Stream, FSDMacroQuery.SQL);
 FDBDataSet.OpenDataSource := frReadBoolean(Stream);
 FSDMacroQuery.RequestLive := frReadBoolean(Stream);
 s := frReadString(Stream);
 if s <> '' then
   FSDMacroQuery.MacroChar := s[1];

 FSDMacroQuery.Filter := frReadString(Stream);
 FSDMacroQuery.Filtered := frReadBoolean(Stream);

 FFixupList['Active'] := frReadBoolean(Stream);

 ReadFields(Stream);
 ReadParams(Stream);
 ReadMacroParams(Stream);
 try
   FSDMacroQuery.Active := FFixupList['Active'];
 except;
 end; { try }

end; { LoadFromStream }

procedure TfrSDMacroQuery.SaveToStream(Stream: TStream);
begin

 LVersion := 1;
 inherited SaveToStream(Stream);

 frWriteString(Stream, FSDMacroQuery.DatabaseName);
 frWriteString(Stream, FSDMacroQuery.SessionName);
 frWriteBoolean(Stream,FSDMacroQuery.AutoCalcFields);
 frWriteString(Stream, Prop['DataSource']);
 frWriteBoolean(Stream,(doDetachOnFetchAll in FSDMacroQuery.Options));
 frWriteBoolean(Stream,FSDMacroQuery.ParamCheck);
 frWriteMemo(Stream, FSDMacroQuery.SQL);
 frWriteBoolean(Stream, FDBDataSet.OpenDataSource);
 frWriteBoolean(Stream, FSDMacroQuery.RequestLive);
 frWriteString(Stream, FSDMacroQuery.MacroChar);
 frWriteString(Stream, FSDMacroQuery.Filter);
 frWriteBoolean(Stream, FSDMacroQuery.Filtered);
 frWriteBoolean(Stream, FSDMacroQuery.Active);
 WriteFields(Stream);
 WriteParams(Stream);
 WriteMacroParams(Stream);

end; { SaveToStream }

procedure TfrSDMacroQuery.Loaded;
begin
 Prop['DataSource'] := FFixupList['DataSource'];
 inherited Loaded;
end; { Loaded }

procedure TfrSDMacroQuery.SQLEditor(Sender: TObject);
begin
 with TfrDBSQLEditorForm.Create(nil) do begin
   Query := FSDMacroQuery;
   M1.Lines.Assign(FSDMacroQuery.SQL);
   if (ShowModal = mrOk) and ((Restrictions and frrfDontModify) = 0) then begin
     FSDMacroQuery.SQL := M1.Lines;
     frDesigner.Modified := True;
   end; { if }
   Free;
 end; { with }
end; { SQLEditor }

procedure TfrSDMacroQuery.ParamsEditor(Sender: TObject);
var
 Params      :TParams;
 ParamValues :TfrVariables;

begin

 if FSDMacroQuery.Params.Count > 0 then begin
   Params := TParams.Create(nil);
   Params.Assign(FSDMacroQuery.Params);
   ParamValues := TfrVariables.Create;
   ParamValues.Assign(FParams);
   with TfrSDParamsForm.Create(nil) do begin
     QueryComp := TfrSDQuery(Self);
     Query := FSDMacroQuery;
     Caption := Self.Name + ' ' + frLoadStr(SParams);
     ParamTypeCB.Visible := False;
     ParamTypeLabel.Visible := False;
     if ShowModal = mrOk then
       frDesigner.Modified := True
     else begin
       FSDMacroQuery.Params.Assign(Params);
       FParams.Assign(ParamValues);
     end; { if }
     Free;
   end; { with }
   Params.Free;
   ParamValues.Free;
 end; { if }

end; { ParamsEditor }

function TfrSDMacroQuery.GetParamKind(Index: Integer): TfrParamKind;
begin

 Result := pkValue;
 if not FSDMacroQuery.Params[Index].Bound then
   Result := pkAssignFromMaster;

end; { GetParamKind }

procedure TfrSDMacroQuery.SetParamKind(Index: Integer; Value: TfrParamKind);
begin

 if Value = pkAssignFromMaster then begin
   FSDMacroQuery.Params[Index].Bound := False;
   FParams.Delete(FParams.IndexOf(FSDMacroQuery.Params[Index].Name));
 end else begin
   FSDMacroQuery.Params[Index].Clear;
   FSDMacroQuery.Params[Index].Bound := True;
   FParams[FSDMacroQuery.Params[Index].Name] := '';
 end; { if }

end; { SetParamKind }

function TfrSDMacroQuery.GetParamText(Index: Integer): String;
begin

 Result := '';
 if ParamKind[Index] = pkValue then
   Result := FParams[FSDMacroQuery.Params[Index].Name];

end; { GetParamText }

procedure TfrSDMacroQuery.SetParamText(Index: Integer; Value: String);
begin
 if ParamKind[Index] = pkValue then
   FParams[FSDMacroQuery.Params[Index].Name] := Value;
end; { SetParamText }

procedure TfrSDMacroQuery.ReadParams(Stream: TStream);
var
  i: Integer;
  w, n: Word;

begin

 Stream.Read(n, 2);
 for i := 0 to n - 1 do
   with FSDMacroQuery.Params[i] do begin
     Stream.Read(w, 2);
     DataType := SDDataTypes[w];
     Stream.Read(w, 2);
     ParamKind[i] := TfrParamKind(w);
     ParamText[i] := frReadString(Stream);
   end; { with }

end; { ReadParams }

procedure TfrSDMacroQuery.WriteParams(Stream: TStream);
var
  i: Integer;
  w: Word;

begin

 w := FSDMacroQuery.Params.Count;
 Stream.Write(w, 2);

 for i := 0 to FSDMacroQuery.Params.Count - 1 do
   with FSDMacroQuery.Params[i] do begin
     for w := 0 to 10 do
       if DataType = SDDataTypes[w] then break;
     Stream.Write(w, 2);
     w := Word(ParamKind[i]);
     Stream.Write(w, 2);
     frWriteString(Stream, ParamText[i]);
   end; { with }

end; { WriteParams }

function TfrSDMacroQuery.GetMacroParamText(Index: Integer): String;
begin
 if VarIsNull(FMacroParams[FSDMacroQuery.Macros[Index].Name]) then
   Result := ''
 else
   Result := FMacroParams[FSDMacroQuery.Macros[Index].Name];
end; { GetMacroParamText }

procedure TfrSDMacroQuery.SetMacroParamText(Index: Integer; const Value: String);
begin
 FMacroParams[FSDMacroQuery.Macros[Index].Name] := Value;
end; { SetMacroParamText }

procedure TfrSDMacroQuery.ReadMacroParams(Stream: TStream);
var
  i: Integer;
  w, n: Word;

begin

 Stream.Read(n, 2);
 for i := 0 to n - 1 do
   with FSDMacroQuery.Macros[i] do begin
     Stream.Read(w, 2);
     DataType := SDDataTypes[w];
     MacroParamText[i] := frReadString(Stream);
   end; { with }

end; { ReadMacroParams }

procedure TfrSDMacroQuery.WriteMacroParams(Stream: TStream);
var
  i: Integer;
  w: Word;

begin

 w := FSDMacroQuery.Macros.Count;
 Stream.Write(w, 2);

 for i := 0 to FSDMacroQuery.Macros.Count - 1 do
   with FSDMacroQuery.Macros[i] do begin
     for w := 0 to 10 do
       if DataType = SDDataTypes[w] then Break;
     Stream.Write(w, 2);
     frWriteString(Stream, MacroParamText[i]);
   end; { with }

end; { WriteMacroParams }

procedure TfrSDMacroQuery.MacroParamsEditor(Sender: TObject);
var
 Params      :TParams;
 ParamValues :TfrVariables;

begin
  
 if FSDMacroQuery.Macros.Count > 0 then begin
   Params := TParams.Create(nil);
   Params.Assign(FSDMacroQuery.Macros);
   ParamValues := TfrVariables.Create;
   ParamValues.Assign(FMacroParams);
   with TfrSDMacroParamsForm.Create(nil) do begin
     QueryComp := Self;
     Query := FSDMacroQuery;
     Caption := Self.Name + ' ' + frLoadStr(SParams);
     if ShowModal = mrOk then
       frDesigner.Modified := True
     else begin
       FSDMacroQuery.Macros.Assign(Params);
       FMacroParams.Assign(ParamValues);
     end; { if }
     Free;
   end; { with }
   Params.Free;
   ParamValues.Free;
 end; { if }

end; { MacroParamsEditor }

procedure TfrSDMacroQuery.BeforeOpenQuery(DataSet: TDataSet);
var
  i: Integer;
  SaveView: TfrView;
  SavePage: TfrPage;
  SaveBand: TfrBand;

 function DefParamValue(Param: TParam): String;
 begin
   if Param.DataType in [ftDate, ftDateTime] then
     Result := '01.01.00'
   else if Param.DataType = ftTime then
     Result := '00:00'
   else
     Result := '0';
 end; { DefParamValue }

begin

 SaveView := CurView;
 CurView := nil;
 SavePage := CurPage;
 CurPage := ParentPage;
 SaveBand := CurBand;
 CurBand := nil;

 i := 0;
 try
   while i < FSDMacroQuery.Params.Count do begin
     if ParamKind[i] = pkValue then
       if DocMode = dmPrinting then
         FSDMacroQuery.Params[i].Text := frParser.Calc(ParamText[i])
       else
         FSDMacroQuery.Params[i].Text := DefParamValue(FSDMacroQuery.Params[i]);
     Inc(i);
   end; { with }
 except
   Memo.Clear;
   Memo.Add(ParamText[i]);
   CurView := Self;
   raise;
 end; { try }

 i := 0;
 try
   while i < FSDMacroQuery.Macros.Count do begin
     if DocMode = dmPrinting then
       FSDMacroQuery.Macros[i].Text := frParser.Calc(MacroParamText[i])
     else
       FSDMacroQuery.Macros[i].Text := DefParamValue(FSDMacroQuery.Macros[i]);
     Inc(i);
   end; { with }
 except
   Memo.Clear;
   Memo.Add(MacroParamText[i]);
   CurView := Self;
   raise;
 end; { try }

 CurView := SaveView;
 CurPage := SavePage;
 CurBand := SaveBand;

end; { BeforeOpenQuery }

procedure TfrSDMacroQuery.SaveToFR3Stream(Stream: TStream);

  procedure WriteStr(const s: String);
  begin
   Stream.Write(s[1], Length(s));
  end; { WriteStr }

  procedure WriteParams(oParams :TParams; cParamsName :String);
  var
    i  :Integer;
    wr :TWriter;
    ms :TMemoryStream;
    v  :TValueType;
    s  :String;

  begin

    ms := TMemoryStream.Create;
    wr := TWriter.Create(ms, 4096);

    v := vaCollection;
    wr.WriteStr(cParamsName);
    wr.Write(v, SizeOf(v));

    for i := 0 to oParams.Count - 1 do
      with oParams[i] do begin
        wr.WriteListBegin;
        wr.WriteStr('Name');
        wr.WriteString(Name);
        wr.WriteStr('DataType');
        v := vaIdent;
        wr.Write(v, SizeOf(v));
        s := GetEnumName(TypeInfo(TFieldType), Integer(DataType));
        wr.WriteStr(s);
        wr.WriteStr('Expression');

        if cParamsName = 'Params' then
          wr.WriteString(ParamText[i])
        else
          wr.WriteString(MacroParamText[i]);

        wr.WriteListEnd;
      end; { with }

    wr.WriteListEnd;
    wr.Free;
    WriteStr(' Propdata="' + frStreamToString(ms) + '"');
    ms.Free;
  end; { WriteParams }
  
begin

 inherited;

 WriteStr(' SQL.text="' + StrToXML(FSDMacroQuery.SQL.Text) + '"');

 if FSDMacroQuery.ParamCheck then
   WriteStr(' ParamCheck="True"');

 if FSDMacroQuery.RequestLive then
   WriteStr(' RequestLive="True"');

 WriteStr(' Master="'+Prop['DataSource']+'"');

 if FSDMacroQuery.Params.Count > 0 then
   WriteParams(FSDMacroQuery.Params, 'Params');

 if FSDMacroQuery.Macros.Count > 0 then
   WriteParams(FSDMacroQuery.Macros, 'Macros');

end; { SaveToFR3Stream }

var
  Bmp: TBitmap;

initialization
  Bmp := TBitmap.Create;
  Bmp.LoadFromResourceName(hInstance, 'FR_SDMACROQUERYCONTROL');
  frRegisterControl(TfrSDMacroQuery, Bmp, IntToStr(SInsertQuery));

finalization
  frUnRegisterObject(TfrSDMacroQuery);
  Bmp.Free;

end.

