{**********************************************}
{                                              }
{     FastReport 2.5 - SQLDirect components    }
{             Table component                  }
{                                              }
{   Copyright (c) 1997-2007 by Yuri Sheino.    }
{   Copyright (c) 1998-2007 by Tzyganenko A.   }
{ Copyright (c) 2001-2007 by Stalker SoftWare  }
{                                              }
{**********************************************}

unit FR_SDTable;

interface

{$I FR.inc}
{$I FR_SD.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, StdCtrls, Controls, Forms,
  Menus, Dialogs, FR_Class, FR_Pars, DB, FR_SDCommon, FR_DBUtils, SDEngine;

type
  TfrSDTable = class(TfrSDDataSet)
  private
    FSDTable: TSDTable;
  protected
    procedure SetPropValue(Index: String; Value: Variant); override;
    function GetPropValue(Index: String): Variant; override;
    function DoMethod(MethodName: String; Par1, Par2, Par3: Variant): Variant; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure LoadFromStream(Stream: TStream); override;
    procedure SaveToStream(Stream: TStream); override;
    procedure DefineProperties; override;
    procedure Loaded; override;
    procedure SaveToFR3Stream(Stream: TStream); override;
    property Table :TSDTable read FSDTable;
  end;

implementation

uses
  FR_Utils, FR_Const
{$IFDEF Delphi6}
, Variants
{$ENDIF};

{ TfrSDTable }

constructor TfrSDTable.Create;
begin

 inherited Create;

 FSDTable := TSDTable.Create(frDialogForm);
 FDataSet := FSDTable;
 FDataSource.DataSet := FDataSet;

 Component := FSDTable;
 BaseName := 'SDTable';
 Bmp.LoadFromResourceName(hInstance, 'FR_SDTABLE');

end; { Create }

destructor TfrSDTable.Destroy;
begin
 inherited Destroy;
end; { Destroy }

procedure TfrSDTable.DefineProperties;

  function GetIndexNames: String;
  var
    i: Integer;

  begin

    Result := '';
    try
      with FSDTable do
      if (TableName <> '') and (IndexDefs <> nil) then begin
        IndexDefs.Update;
        for i := 0 to IndexDefs.Count - 1 do
          if IndexDefs[i].Name <> '' then
            Result := Result + IndexDefs[i].Name + ';';
      end; { if }
    except;
    end; { try }

  end; { GetIndexNames }

  function GetIndexFieldNames: String;
  var
    I: Integer;

  begin

   Result := '';
   try

     with FSDTable do begin

       if (TableName <> '') and (IndexDefs <> nil) then begin

         IndexDefs.Update;
         for I := 0 to IndexDefs.Count - 1 do
           with IndexDefs[I] do
             if (Options * [ixExpression, ixDescending] = []) and (Fields <> '') then
               Result := Result + Fields + ';';

       end; { if }
       
     end; { with }

   except;
   end; { try }

  end; { GetIndexFieldNames }

  function GetTableNames: String;
  var
    i: Integer;
    sl: TStringList;
  begin

   Result := '';
   try

     if FSDTable.DatabaseName <> '' then begin
       sl := TStringList.Create;
       Session.GetTableNames(FSDTable.DatabaseName, '', False, sl);
       sl.Sort;
       for i := 0 to sl.Count - 1 do
         Result := Result + sl[i] + ';';
       sl.Free;
     end; { if }

   except;
   end; { try }

  end; { GetTableNames }


begin

  inherited DefineProperties;

  AddEnumProperty('IndexName', GetIndexNames, [Null]);
  AddEnumProperty('IndexFieldNames', GetIndexFieldNames, [Null]);
  AddEnumProperty('TableName', GetTableNames, [Null]);
  AddProperty('ReadOnly', [frdtBoolean], nil);
  AddProperty('DefaultIndex', [frdtBoolean], nil);

end; { DefineProperties }

procedure TfrSDTable.SetPropValue(Index: String; Value: Variant);
begin

 inherited SetPropValue(Index, Value);
 Index := AnsiUpperCase(Index);

 if Index = 'TABLENAME' then
   FSDTable.TableName := Value
 else
 if Index = 'INDEXNAME' then
   FSDTable.IndexName := Value
 else
 if Index = 'INDEXFIELDNAMES' then
   FSDTable.IndexFieldNames := Value
 else
 if Index = 'READONLY' then
   FSDTable.ReadOnly := Value
 else
 if Index = 'DEFAULTINDEX' then
   FSDTable.DefaultIndex := Value

end; { SetPropValue }

function TfrSDTable.GetPropValue(Index: String): Variant;
begin

 Index := AnsiUpperCase(Index);
 Result := inherited GetPropValue(Index);
 if Result <> Null then Exit;

 if Index = 'TABLENAME' then
   Result := FSDTable.TableName
 else
 if Index = 'INDEXNAME' then
   Result := FSDTable.IndexName
 else
 if Index = 'INDEXFIELDNAMES' then
   Result := FSDTable.IndexFieldNames
 else
 if Index = 'READONLY' then
   Result := FSDTable.ReadOnly
 else
 if Index = 'DEFAULTINDEX' then
   Result := FSDTable.DefaultIndex

end; { GetPropValue }

function TfrSDTable.DoMethod(MethodName: String; Par1, Par2, Par3: Variant): Variant;
begin

 Result := inherited DoMethod(MethodName, Par1, Par2, Par3);

end; { DoMethod }

procedure TfrSDTable.LoadFromStream(Stream: TStream);
begin

 FFixupList.Clear;
 inherited LoadFromStream(Stream);

 FSDTable.DatabaseName := frReadString(Stream);
 FSDTable.SessionName := frReadString(Stream);
 FSDTable.AutoCalcFields := frReadBoolean(Stream);

 if frReadBoolean(Stream) then
   FSDTable.Options := FSDTable.Options+[doDetachOnFetchAll]
 else
   FSDTable.Options := FSDTable.Options-[doDetachOnFetchAll];

 FDBDataSet.OpenDataSource := frReadBoolean(Stream);
 FSDTable.IndexName := frReadString(Stream);
 FSDTable.TableName := frReadString(Stream);
 FSDTable.DefaultIndex := frReadBoolean(Stream);
 FSDTable.ReadOnly := frReadBoolean(Stream);
 FSDTable.Filter := frReadString(Stream);
 FSDTable.Filtered := frReadBoolean(Stream);

 FFixupList['Active'] := frReadBoolean(Stream);

 ReadFields(Stream);
 try
   FSDTable.Active := FFixupList['Active'];
 except;
 end; { try }

end; { LoadFromStream }

procedure TfrSDTable.SaveToStream(Stream: TStream);
begin

 LVersion := 1;
 inherited SaveToStream(Stream);

 frWriteString(Stream, FSDTable.DatabaseName);
 frWriteString(Stream, FSDTable.SessionName);
 frWriteBoolean(Stream,FSDTable.AutoCalcFields);
 frWriteBoolean(Stream,(doDetachOnFetchAll in FSDTable.Options));
 frWriteBoolean(Stream, FDBDataSet.OpenDataSource);
 frWriteString(Stream, FSDTable.IndexName);
 frWriteString(Stream, FSDTable.TableName);
 frWriteBoolean(Stream, FSDTable.DefaultIndex);
 frWriteBoolean(Stream, FSDTable.ReadOnly);
 frWriteString(Stream, FSDTable.Filter);
 frWriteBoolean(Stream, FSDTable.Filtered);
 frWriteBoolean(Stream, FSDTable.Active);
 WriteFields(Stream);

end; { SaveToStream }

procedure TfrSDTable.Loaded;
begin
 inherited Loaded;
end; { Loaded }

procedure TfrSDTable.SaveToFR3Stream(Stream: TStream);

 procedure WriteStr(const s: String);
 begin
  Stream.Write(s[1], Length(s));
 end;

begin

 inherited;

  WriteStr(' IndexName="' + StrToXML(FSDTable.IndexName) + '"');
  WriteStr(' IndexFieldNames="' + StrToXML(FSDTable.IndexFieldNames) + '"');
  WriteStr(' TableName="' + StrToXML(FSDTable.TableName) + '"');

 if FSDTable.ReadOnly then
   WriteStr(' ReadOnly="True"',);

 if FSDTable.DefaultIndex then
   WriteStr(' DefaultIndex="True"',);

end; { SaveToFR3Stream }

var
  Bmp: TBitmap;

initialization
  Bmp := TBitmap.Create;
  Bmp.LoadFromResourceName(hInstance, 'FR_SDTABLECONTROL');
  frRegisterControl(TfrSDTable, Bmp, IntToStr(SInsertTable));

finalization
  frUnRegisterObject(TfrSDTable);
  Bmp.Free;

end.

